/**
  ******************************************************************************
  * @file    LwIP/LwIP_HTTP_Server_Netconn_RTOS/Src/httpser-netconn.c 
  * @author  Yannick Avelino - Electrolab
  * @brief   Basic telnet-like CLI server using LwIP netconn API
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; CC BY SA - 2018 Electrolab.</center></h2>
  *
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "lwip/apps/fs.h"
#include "string.h"
#include "cliserver-netconn.h"
#include "cmsis_os.h"
#include "main.h"

#include "io_manage.h"

#include <stdio.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define CLISERVER_THREAD_PRIO    ( osPriorityAboveNormal )

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

char CLI_buffer[CLI_BUFFER_LENGTH];  //CLI circular buffer and indexes
u16_t CLI_buffer_Write_Index=0;
u16_t CLI_buffer_Seek_Index=0;
char CLI_out_buffer[CLI_BUFFER_LENGTH];  //CLI out buffer

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
static void CLI_server_serve(struct netconn *conn);


/**
  * @brief serve tcp connection  
  * @param conn: pointer on connection structure 
  * @retval None
  */
static void CLI_interpreter(struct netconn *conn)
{
  int param;
  int out_length;

  //Check the request...
  if (strncmp(CLI_buffer, "GETVER", 6) == 0)
   {
	 /*Generate success message*/
     out_length=snprintf(CLI_out_buffer,CLI_BUFFER_LENGTH, "<OK%02X%02X> VERSION %d REV %d\r\n",SOFT_VER, SOFT_REV,SOFT_VER, SOFT_REV);
     netconn_write(conn, CLI_out_buffer, out_length, NETCONN_COPY);
   }
  else if (strncmp(CLI_buffer, "SETFAN", 6) == 0)
   {
   /* Extract parameter */
   CLI_buffer[CLI_buffer_Write_Index]=0;
   param=atoi(&CLI_buffer[6]);
   /* if parameter within range, process, else, generate an error msg*/
   if ((param>=0) && (param<=100))
     {
	 /*Update output*/
	 /*TODO*/
	 io_set_PWM1_duty(param);
	 io_set_PWM2_duty(param);
	 io_set_PWM3_duty(param);
	 io_set_PWM4_duty(param);


	 /*Generate success message*/
     out_length=snprintf(CLI_out_buffer,CLI_BUFFER_LENGTH, "<OK> FAN RUNNING AT %d%%\r\n",param);
     }
   else
     {
	 out_length=snprintf(CLI_out_buffer,CLI_BUFFER_LENGTH, "<N03> PARAMETER OUT OF RANGE\r\n");
     }

   netconn_write(conn, CLI_out_buffer, out_length, NETCONN_COPY);
   }
  else if (strncmp(CLI_buffer, "SETAUX", 6) == 0)
   {
   /* Extract parameter */
   CLI_buffer[CLI_buffer_Write_Index]=0;
   param=atoi(&CLI_buffer[6]);
   /* if parameter within range, process, else, generate an error msg*/
   if ((param>=0) && (param<=1))
      {
	  /*Update output*/
      /*TODO*/
	  io_set_aux(param);

      /*Generate success message*/
	  out_length=snprintf(CLI_out_buffer,CLI_BUFFER_LENGTH, "<OK> AUX OUT = %d\r\n",param);
	  }
   else
	  {
      out_length=snprintf(CLI_out_buffer,CLI_BUFFER_LENGTH, "<N03> PARAMETER OUT OF RANGE\r\n");
	  }


	  netconn_write(conn, CLI_out_buffer, out_length, NETCONN_COPY);
	  //netconn_write(conn, "<AUX STATUS x>\r\n", strlen("<AUX STATUS x>\r\n"), NETCONN_COPY);
   }
  else if (strncmp(CLI_buffer, "STATUS", 6) == 0)
   {


	  netconn_write(conn, "<N02>\r\n", strlen("<N02>\r\n"), NETCONN_COPY);
   }
  else if (strncmp(CLI_buffer, "TASKS", 5) == 0)
   {


	  netconn_write(conn, "<N02>\r\n", strlen("<N02>\r\n"), NETCONN_COPY);
   }
  else
    {//What the hell man !?!
	  netconn_write(conn, "<N01>\r\n", strlen("<N01>\r\n"), NETCONN_COPY);
    }
  CLI_buffer_Write_Index=0;
}


/**
  * @brief serve tcp connection
  * @param conn: pointer on connection structure
  * @retval None
  */
static void CLI_server_serve(struct netconn *conn)
{
  struct netbuf *inbuf;
  //err_t recv_err;
  char* buf;
  u16_t buflen;
  u16_t i;

  //struct fs_file file;
  
  /* Read the data from the port, blocking if nothing yet there. 
   We assume the request (the part we care about) is in one netbuf */
  while (netconn_recv(conn, &inbuf) == ERR_OK)
  {
    if (netconn_err(conn) == ERR_OK) 
    {
      //get input buffer
      netbuf_data(inbuf, (void**)&buf, &buflen);


      if(buflen!=0)
        {
    	//copy to input rolling CLI buffer
    	for (i=0;i<(buflen-1);i++)
    	  {
          /*Drop spaces*/
          if (buf[i]!=' ')
            {
    	    /*Convert to upper case if needed*/
    	    if ((buf[i]>='a') && (buf[i]<='z'))
    		    CLI_buffer[CLI_buffer_Write_Index]=(buf[i]-32);
    	    else
    		    CLI_buffer[CLI_buffer_Write_Index]=buf[i];

            CLI_buffer_Write_Index=(CLI_buffer_Write_Index+1)%CLI_BUFFER_LENGTH;
    	    }
    	  }

    	//process buffer into CLI interpreter if last received char is a CR
    	if(CLI_buffer_Write_Index>0)
    	  {
    	  if (CLI_buffer[CLI_buffer_Write_Index-1]==13)
    	    CLI_interpreter(conn);
    	  }

        }

      //delete
      netbuf_delete(inbuf);



    }
  }

  
  /* Delete the buffer (netconn_recv gives us ownership,
   so we have to make sure to deallocate the buffer) */

}


/**
  * @brief  http server thread 
  * @param arg: pointer on argument(not used here) 
  * @retval None
  */
static void CLI_server_netconn_thread(void *arg)
{ 
  struct netconn *conn, *newconn;
  err_t err, accept_err;
  
  /* Create a new TCP connection handle */
  conn = netconn_new(NETCONN_TCP);
  
  if (conn!= NULL)
  {
    /* Bind to port 24 (TELNET) with default IP address */
    err = netconn_bind(conn, NULL, 24);
    
    if (err == ERR_OK)
    {
      /* Put the connection into LISTEN state */
      netconn_listen(conn);
  
      while(1) 
      {
        /* accept any incoming connection */
        accept_err = netconn_accept(conn, &newconn);
        if(accept_err == ERR_OK)
        {
          /* serve connection */
          CLI_buffer_Write_Index=0;
          CLI_buffer_Seek_Index=0;
          CLI_server_serve(newconn);

          /* If server returns, close connection*/
          netconn_close(newconn);

          /* delete connection */
          netconn_delete(newconn);
        }
      }
    }
  }
}

/**
  * @brief  Initialize the HTTP server (start its thread) 
  * @param  none
  * @retval None
  */
void CLI_server_netconn_init()
{
  sys_thread_new("CLI", CLI_server_netconn_thread, NULL, DEFAULT_THREAD_STACKSIZE, CLISERVER_THREAD_PRIO);
}

/**
  * @brief  Create and send a dynamic Web Page. This page contains the list of 
  *         running tasks and the number of page hits. 
  * @param  conn pointer on connection structure 
  * @retval None
  */
void DynCLI(void)
{
  /*portCHAR PAGE_BODY[512];

  memset(PAGE_BODY, 0,512);

  strcat((char *)PAGE_BODY, "Name          State  Priority  Stack   Num" );
  strcat((char *)PAGE_BODY, "------------------------------------------");*/
    
  /* The list of tasks and their status */
  /*osThreadList((unsigned char *)(PAGE_BODY + strlen(PAGE_BODY)));
  strcat((char *)PAGE_BODY, "------------------------------------------");
  strcat((char *)PAGE_BODY, "B : Blocked, R : Ready, D : Deleted, S : Suspended");*/

  /* Send the dynamically generated page */
  //netconn_write(conn, PAGE_START, strlen((char*)PAGE_START), NETCONN_COPY);
  //netconn_write(conn, PAGE_BODY, strlen(PAGE_BODY), NETCONN_COPY);
}
