/**
  ******************************************************************************
  * @file    HeatAck/Src/io_manage.c
  * @author  Electrolab
  * @brief   This file implements external world management
  ******************************************************************************
  * @attention
  *
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "stm32f2xx_hal.h"
#include "stm32f2xx_nucleo_144.h"
#include "io_manage.h"
#include <string.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
//#define  PERIOD_VALUE       (uint32_t)(666 - 1)  /* Period Value  */
#define  PERIOD_VALUE       (uint32_t)(60000 - 1)  /* Period Value  */
//4 -> 15MHz
//666 -> 0.090MHz

#define  PWM1_DEFVALUE       100        /* Capture Compare 1 Value  */
//#define  PULSE2_VALUE       (uint32_t)(PERIOD_VALUE*37.5/100) /* Capture Compare 2 Value  */
#define  PWM2_DEFVALUE       100
//#define  PULSE3_VALUE       (uint32_t)(PERIOD_VALUE/4)        /* Capture Compare 3 Value  */
#define  PWM3_DEFVALUE       100
//#define  PULSE4_VALUE       (uint32_t)(PERIOD_VALUE*12.5/100) /* Capture Compare 4 Value  */
#define  PWM4_DEFVALUE       100

TIM_OC_InitTypeDef sConfig;
TIM_HandleTypeDef    TimPWMHandle;
//uint32_t PWM_duty[4];

/* Private functions ---------------------------------------------------------*/
/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
static void Error_Handler(void)
{
  /* Turn LED3 on */
  BSP_LED_On(LED2);
  while (1)
  {
  }
}

/* Public functions ----------------------------------------------------------*/

int io_set_PWM1_duty (uint32_t duty)
{

sConfig.Pulse = PERIOD_VALUE*duty/100;
HAL_TIM_PWM_Stop(&TimPWMHandle, TIM_CHANNEL_1);
if (HAL_TIM_PWM_ConfigChannel(&TimPWMHandle, &sConfig, TIM_CHANNEL_1) != HAL_OK)
    {
	/* Configuration Error */
	return HAL_ERROR;
    }
HAL_TIM_PWM_Start(&TimPWMHandle, TIM_CHANNEL_1);
return HAL_OK;
}
/* Public functions ----------------------------------------------------------*/
int io_set_PWM2_duty (uint32_t duty)
{

sConfig.Pulse = PERIOD_VALUE*duty/100;;
HAL_TIM_PWM_Stop(&TimPWMHandle, TIM_CHANNEL_2);
if (HAL_TIM_PWM_ConfigChannel(&TimPWMHandle, &sConfig, TIM_CHANNEL_2) != HAL_OK)
    {
	/* Configuration Error */
	return HAL_ERROR;
    }
HAL_TIM_PWM_Start(&TimPWMHandle, TIM_CHANNEL_2);
return HAL_OK;
}
/* Public functions ----------------------------------------------------------*/

int io_set_PWM3_duty (uint32_t duty)
{

sConfig.Pulse = PERIOD_VALUE*duty/100;
HAL_TIM_PWM_Stop(&TimPWMHandle, TIM_CHANNEL_3);
if (HAL_TIM_PWM_ConfigChannel(&TimPWMHandle, &sConfig, TIM_CHANNEL_3) != HAL_OK)
    {
	/* Configuration Error */
	return HAL_ERROR;
    }
HAL_TIM_PWM_Start(&TimPWMHandle, TIM_CHANNEL_3);
return HAL_OK;
}

/* Public functions ----------------------------------------------------------*/
int io_set_PWM4_duty (uint32_t duty)
{

sConfig.Pulse = PERIOD_VALUE*duty/100;
HAL_TIM_PWM_Stop(&TimPWMHandle, TIM_CHANNEL_4);
if (HAL_TIM_PWM_ConfigChannel(&TimPWMHandle, &sConfig, TIM_CHANNEL_4) != HAL_OK)
    {
	/* Configuration Error */
	return HAL_ERROR;
    }
HAL_TIM_PWM_Start(&TimPWMHandle, TIM_CHANNEL_4);
return HAL_OK;
}

/* Public functions ----------------------------------------------------------*/
int io_initialize (void)
{
// configure input compare


//configure PWM

    /*##-1- Configure the TIM peripheral #######################################*/
      /* -----------------------------------------------------------------------
      TIM3 Configuration: generate 4 PWM signals with 4 different duty cycles.

        In this example TIM3 input clock (TIM3CLK) is set to APB1 clock x 2,
        since APB1 prescaler is equal to 2.
          TIM3CLK = APB1CLK*2
          APB1CLK = HCLK/4
          => TIM3CLK = HCLK/2 = SystemCoreClock/2

        To get TIM3 counter clock at 10 MHz, the prescaler is computed as follows:
           Prescaler = (TIM3CLK / TIM3 counter clock) - 1
           Prescaler = ((SystemCoreClock / 2) /10 MHz) - 1

        To get TIM3 output clock at 15 KHz, the period (ARR)) is computed as follows:
           ARR = (TIM3 counter clock / TIM3 output clock) - 1
               = 665

        TIM3 Channel1 duty cycle = (TIM3_CCR1/ TIM3_ARR + 1)* 100 = 50%
        TIM3 Channel2 duty cycle = (TIM3_CCR2/ TIM3_ARR + 1)* 100 = 37.5%
        TIM3 Channel3 duty cycle = (TIM3_CCR3/ TIM3_ARR + 1)* 100 = 25%
        TIM3 Channel4 duty cycle = (TIM3_CCR4/ TIM3_ARR + 1)* 100 = 12.5%

        Note:
         SystemCoreClock variable holds HCLK frequency and is defined in system_stm32f2xx.c file.
         Each time the core clock (HCLK) changes, user had to update SystemCoreClock
         variable value. Otherwise, any configuration based on this variable will be incorrect.
         This variable is updated in three ways:
          1) by calling CMSIS function SystemCoreClockUpdate()
          2) by calling HAL API function HAL_RCC_GetSysClockFreq()
          3) each time HAL_RCC_ClockConfig() is called to configure the system clock frequency
      ----------------------------------------------------------------------- */

      /* Initialize TIMx peripheral as follows:
           + Prescaler = (SystemCoreClock / 2 / 10000000) - 1
           + Period = (666 - 1)
           + ClockDivision = 0
           + Counter direction = Up
      */
      TimPWMHandle.Instance               = TIMx;
      TimPWMHandle.Init.Prescaler         = 0;
      TimPWMHandle.Init.Period            = PERIOD_VALUE;
      TimPWMHandle.Init.ClockDivision     = 0;
      TimPWMHandle.Init.CounterMode       = TIM_COUNTERMODE_UP;
      TimPWMHandle.Init.RepetitionCounter = 0;
      TimPWMHandle.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
      if (HAL_TIM_PWM_Init(&TimPWMHandle) != HAL_OK)
      {
        /* Initialization Error */
        Error_Handler();
      }

      /*##-2- Configure the PWM channels #########################################*/
      /* Common configuration for all channels */
      sConfig.OCMode       = TIM_OCMODE_PWM1;
      sConfig.OCPolarity   = TIM_OCPOLARITY_HIGH;
      sConfig.OCFastMode   = TIM_OCFAST_DISABLE;
      sConfig.OCNPolarity  = TIM_OCNPOLARITY_HIGH;
      sConfig.OCNIdleState = TIM_OCNIDLESTATE_RESET;
      sConfig.OCIdleState  = TIM_OCIDLESTATE_RESET;

      /* Set the pulse value for channel 1 */
      if (io_set_PWM1_duty(PWM1_DEFVALUE) !=HAL_OK)
	      {
	      /* Configuration Error */
	      Error_Handler();
	      }
      /* Set the pulse value for channel 2 */
	  if (io_set_PWM2_duty(PWM2_DEFVALUE) !=HAL_OK)
	      {
	      /* Configuration Error */
	      Error_Handler();
	      }
      /* Set the pulse value for channel 3 */
	  if (io_set_PWM3_duty(PWM3_DEFVALUE) !=HAL_OK)
		  {
		  /* Configuration Error */
		  Error_Handler();
		  }
      /* Set the pulse value for channel 4 */
      if (io_set_PWM4_duty(PWM4_DEFVALUE) !=HAL_OK)
          {
          /* Configuration Error */
          Error_Handler();
          }



}

int io_set_aux(int state)
{
//as a first test : BSP toggle LED 3 (the blue one)
	if (state==0)
		BSP_LED_Off(LED3); //close the valve
	else
		BSP_LED_On(LED3); //open the valve
}



//  osSemaphoreRelease(s_xSemaphore);
/* create a binary semaphore used for informing ethernetif of frame reception */
//  osSemaphoreDef(SEM);
//  HAL_GetTick();
/************************ (C) COPYLEFT Electrolab *****END OF FILE****/
