/**
  ******************************************************************************
  * @file    LwIP/LwIP_HTTP_Server_Netconn_RTOS/Src/httpser-netconn.c 
  * @author  Yannick Avelino - Electrolab
  * @brief   Basic telnet-like CLI server using LwIP netconn API
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; CC BY SA - 2018 Electrolab.</center></h2>
  *
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "lwip/apps/fs.h"
#include "string.h"
#include "cliserver-netconn.h"
#include "cmsis_os.h"

#include <stdio.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define CLISERVER_THREAD_PRIO    ( osPriorityAboveNormal )

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/




/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
static void CLI_server_serve(struct netconn *conn);

/**
  * @brief serve tcp connection  
  * @param conn: pointer on connection structure 
  * @retval None
  */
static void CLI_server_serve(struct netconn *conn)
{
  struct netbuf *inbuf;
  err_t recv_err;
  char* buf;
  u16_t buflen;
  //struct fs_file file;
  
  /* Read the data from the port, blocking if nothing yet there. 
   We assume the request (the part we care about) is in one netbuf */
  while (netconn_recv(conn, &inbuf) == ERR_OK)
  {
    if (netconn_err(conn) == ERR_OK) 
    {
      //get input buffer
      netbuf_data(inbuf, (void**)&buf, &buflen);

      //copy to input rolling CLI buffer

      //delete
      netbuf_delete(inbuf);

      //process CLI buffer

      //answer
      //netconn_write(conn, "Hello world", strlen("Hello world"), NETCONN_COPY);
      netconn_write(conn, buf, buflen, NETCONN_COPY);
    }
  }

  
  /* Delete the buffer (netconn_recv gives us ownership,
   so we have to make sure to deallocate the buffer) */

}


/**
  * @brief  http server thread 
  * @param arg: pointer on argument(not used here) 
  * @retval None
  */
static void CLI_server_netconn_thread(void *arg)
{ 
  struct netconn *conn, *newconn;
  err_t err, accept_err;
  
  /* Create a new TCP connection handle */
  conn = netconn_new(NETCONN_TCP);
  
  if (conn!= NULL)
  {
    /* Bind to port 24 (TELNET) with default IP address */
    err = netconn_bind(conn, NULL, 24);
    
    if (err == ERR_OK)
    {
      /* Put the connection into LISTEN state */
      netconn_listen(conn);
  
      while(1) 
      {
        /* accept any incoming connection */
        accept_err = netconn_accept(conn, &newconn);
        if(accept_err == ERR_OK)
        {
          /* serve connection */
          CLI_server_serve(newconn);

          /* If server returns, close connection*/
          netconn_close(newconn);

          /* delete connection */
          netconn_delete(newconn);
        }
      }
    }
  }
}

/**
  * @brief  Initialize the HTTP server (start its thread) 
  * @param  none
  * @retval None
  */
void CLI_server_netconn_init()
{
  sys_thread_new("CLI", CLI_server_netconn_thread, NULL, DEFAULT_THREAD_STACKSIZE, CLISERVER_THREAD_PRIO);
}

/**
  * @brief  Create and send a dynamic Web Page. This page contains the list of 
  *         running tasks and the number of page hits. 
  * @param  conn pointer on connection structure 
  * @retval None
  */
void DynCLI(void)
{
  /*portCHAR PAGE_BODY[512];

  memset(PAGE_BODY, 0,512);

  strcat((char *)PAGE_BODY, "Name          State  Priority  Stack   Num" );
  strcat((char *)PAGE_BODY, "------------------------------------------");*/
    
  /* The list of tasks and their status */
  /*osThreadList((unsigned char *)(PAGE_BODY + strlen(PAGE_BODY)));
  strcat((char *)PAGE_BODY, "------------------------------------------");
  strcat((char *)PAGE_BODY, "B : Blocked, R : Ready, D : Deleted, S : Suspended");*/

  /* Send the dynamically generated page */
  //netconn_write(conn, PAGE_START, strlen((char*)PAGE_START), NETCONN_COPY);
  //netconn_write(conn, PAGE_BODY, strlen(PAGE_BODY), NETCONN_COPY);
}
